#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright (C) 2013 David Callé <davidc@framli.eu>
# Copyright (C) 2012 Chris Wayne <cwayne@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import Unity, UnityExtras
from gi.repository import Gio, GLib
import urllib.parse
import urllib.request
import json
import gettext

APP_NAME = 'unity-scope-imdb'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Info.Imdb'
UNIQUE_PATH = '/com/canonical/unity/scope/info/imdb'
SEARCH_URI = 'https://sg.media-imdb.com/'
SEARCH_HINT = _('Search iMDB')
NO_RESULTS_HINT = _('Sorry, there are no iMDB results that match your search.')
PROVIDER_CREDITS = _('Powered by iMDB')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR+'service-imdb.svg'
DEFAULT_RESULT_ICON = SVG_DIR+'result-info.svg'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT

c1 = {'id'      :'encyclopedia',
      'name'    :_('iMDB'),
      'icon'    :SVG_DIR+'group-installed.svg',
      'renderer':Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]
FILTERS = []

m1 = {'id'   :'role',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
m2 = {'id'   :'year',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
EXTRA_METADATA = [m1, m2]

previous_search = None
previous_results = None

def search(search, filters):
    '''
    Any search method returning results as a list of tuples.
    Available tuple fields:
    uri (string)
    icon (string)
    title (string)
    comment (string)
    dnd_uri (string)
    mimetype (string)
    category (int)
    result_type (Unity ResultType)
    extras metadata fields (variant)
    '''
    results = []
    global previous_results, previous_search
    if not search:
        return results
    search = urllib.parse.quote(search)
    uri = '%ssuggests/%s/%s.json' % (SEARCH_URI, search[0], search)
    print (uri)
    data = None
    try:
        response = urllib.request.urlopen(uri).read()
        response = response.decode('utf-8').split('(', 1)[1][:-1]
        raw_data = json.loads(response)
    except Exception as error:
        print(error)
        if previous_search and previous_results and search.startswith(previous_search):
            raw_data = previous_results
        else:
            return results
    if not raw_data or len(raw_data['d']) == 0:
        return results
    previous_search = search
    previous_results = raw_data
    for d in raw_data['d']:
        try:
            icon = d['i'][0]
        except:
            icon = ''
        title = d['l']
        for hint in ['d', 'q', 'y', 's']:
            if not hint in d:
                d[hint] = ''
        if d['id'][:2] == 'nm':
            uri = 'http://imdb.com/name/%s' % d['id']
            comment = d['s']
            role = comment.split(', ')[0]
            comment = comment.replace(role+', ', '')
            year = ''
        elif d['id'][:2]:
            url = 'http://imdb.com/title/%s' % d['id']
            role = d['q']
            year = str(d['y'])
            comment = d['s']
        else:
            uri = ''
        if role == 'feature':
            role = ''
        results.append({'uri':uri,
                        'icon':icon,
                        'title':title,
                        'comment':comment,
                        'role':role,
                        'year':year})
    return results


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch (Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super (MySearch, self).__init__()
        self.set_search_context (search_context)

    def do_run (self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                result_set.add_result(**i)
        except Exception as error:
            print (error)

class Preview (Unity.ResultPreviewer):

    def do_run(self):
        im = Gio.FileIcon.new(Gio.file_new_for_uri(self.result.icon_hint))
        preview = Unity.GenericPreview.new(self.result.title, '', im)
        role = self.result.metadata['role'].get_string()
        year = self.result.metadata['year'].get_string()
        if role == '':
            preview.props.subtitle = year
            preview.add_info(Unity.InfoHint.new("starring", _("Starring"), None, self.result.comment))
        elif year == '':
            preview.props.subtitle = role
            preview.add_info(Unity.InfoHint.new("known_for", _("Known for"), None, self.result.comment))
        else:
            preview.props.subtitle = '%s, %s' % (year, role)
            preview.add_info(Unity.InfoHint.new("starring", _("Starring"), None, self.result.comment))
        icon = Gio.FileIcon.new (Gio.file_new_for_path(PROVIDER_ICON))
        view_action = Unity.PreviewAction.new("view", _("View"), icon)
        preview.add_action(view_action)
        return preview

class Scope (Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint (self):
        return SEARCH_HINT

    def do_get_schema (self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new ()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories (self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new ()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new (c['id'], c['name'],
                                          Gio.ThemedIcon.new(c['icon']),
                                          c['renderer'])
                cs.add (cat)
        return cs

    def do_get_filters (self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new ()
#        if FILTERS:
#            
        return fs

    def do_get_group_name (self):
        return GROUP_NAME

    def do_get_unique_name (self):
        return UNIQUE_PATH

    def do_create_search_for_query (self, search_context):
        se = MySearch (search_context)
        return se

    def do_create_previewer(self, result, metadata):
        rp = Preview()
        rp.set_scope_result(result)
        rp.set_search_metadata(metadata)
        return rp

def load_scope():
    return Scope()
